.page {
    padding: 2vw;
    width: 100%;
    display: flex;
    flex-direction: column;
    height: 100%;
}

div.flex-row {
    display: flex;
    justify-content: space-around;
    flex-direction: row;
    flex: 1 1 0;
    overflow-y: auto;
}

div.flex-column {
    display: flex;
    flex-direction: column;
    margin: 2.5vw;
    flex: 1 1 0;
    min-width: 0;
}

.item-container {
    width: 100%;
    height: 100%;
    position: relative;
}
