# coding: utf-8

"""
    OOXML Automation

    This API helps users convert Excel and Powerpoint documents into rich, live dashboards and stories.  # noqa: E501

    The version of the OpenAPI document: 0.1.0-no-tags
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class SharedGradientStops(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'gradient_fills_id': 'str',
        'position': 'int',
        'id': 'str'
    }

    attribute_map = {
        'gradient_fills_id': 'gradientFillsId',
        'position': 'position',
        'id': 'id'
    }

    def __init__(self, gradient_fills_id=None, position=None, id=None):  # noqa: E501
        """SharedGradientStops - a model defined in OpenAPI"""  # noqa: E501

        self._gradient_fills_id = None
        self._position = None
        self._id = None
        self.discriminator = None

        self.gradient_fills_id = gradient_fills_id
        if position is not None:
            self.position = position
        if id is not None:
            self.id = id

    @property
    def gradient_fills_id(self):
        """Gets the gradient_fills_id of this SharedGradientStops.  # noqa: E501


        :return: The gradient_fills_id of this SharedGradientStops.  # noqa: E501
        :rtype: str
        """
        return self._gradient_fills_id

    @gradient_fills_id.setter
    def gradient_fills_id(self, gradient_fills_id):
        """Sets the gradient_fills_id of this SharedGradientStops.


        :param gradient_fills_id: The gradient_fills_id of this SharedGradientStops.  # noqa: E501
        :type: str
        """

        self._gradient_fills_id = gradient_fills_id

    @property
    def position(self):
        """Gets the position of this SharedGradientStops.  # noqa: E501


        :return: The position of this SharedGradientStops.  # noqa: E501
        :rtype: int
        """
        return self._position

    @position.setter
    def position(self, position):
        """Sets the position of this SharedGradientStops.


        :param position: The position of this SharedGradientStops.  # noqa: E501
        :type: int
        """

        self._position = position

    @property
    def id(self):
        """Gets the id of this SharedGradientStops.  # noqa: E501


        :return: The id of this SharedGradientStops.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this SharedGradientStops.


        :param id: The id of this SharedGradientStops.  # noqa: E501
        :type: str
        """

        self._id = id

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, SharedGradientStops):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other
