# coding: utf-8

"""
    OOXML Automation

    This API helps users convert Excel and Powerpoint documents into rich, live dashboards and stories.  # noqa: E501

    The version of the OpenAPI document: 0.1.0-no-tags
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class SlideShapeTreesDetails(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'slide_id': 'str',
        'slide': 'SlideSlidesDetails',
        'group_elements': 'list[SlideGroupElementsDetails]',
        'group_element_id': 'str',
        'group_element': 'SlideGroupElementsDetails',
        'hidden': 'bool',
        'title': 'str',
        'ooxml_id': 'int',
        'svg_blob_url': 'str',
        'base_element_blob_url': 'str',
        'changed_base_element_blob_url': 'str',
        'package_uri': 'str',
        'name': 'str',
        'id': 'str',
        'date_created': 'datetime',
        'user_created': 'str',
        'date_modified': 'datetime',
        'user_modified': 'str'
    }

    attribute_map = {
        'slide_id': 'slideId',
        'slide': 'slide',
        'group_elements': 'groupElements',
        'group_element_id': 'groupElementId',
        'group_element': 'groupElement',
        'hidden': 'hidden',
        'title': 'title',
        'ooxml_id': 'ooxmlId',
        'svg_blob_url': 'svgBlobUrl',
        'base_element_blob_url': 'baseElementBlobUrl',
        'changed_base_element_blob_url': 'changedBaseElementBlobUrl',
        'package_uri': 'packageUri',
        'name': 'name',
        'id': 'id',
        'date_created': 'dateCreated',
        'user_created': 'userCreated',
        'date_modified': 'dateModified',
        'user_modified': 'userModified'
    }

    def __init__(self, slide_id=None, slide=None, group_elements=None, group_element_id=None, group_element=None, hidden=None, title=None, ooxml_id=None, svg_blob_url=None, base_element_blob_url=None, changed_base_element_blob_url=None, package_uri=None, name=None, id=None, date_created=None, user_created=None, date_modified=None, user_modified=None):  # noqa: E501
        """SlideShapeTreesDetails - a model defined in OpenAPI"""  # noqa: E501

        self._slide_id = None
        self._slide = None
        self._group_elements = None
        self._group_element_id = None
        self._group_element = None
        self._hidden = None
        self._title = None
        self._ooxml_id = None
        self._svg_blob_url = None
        self._base_element_blob_url = None
        self._changed_base_element_blob_url = None
        self._package_uri = None
        self._name = None
        self._id = None
        self._date_created = None
        self._user_created = None
        self._date_modified = None
        self._user_modified = None
        self.discriminator = None

        self.slide_id = slide_id
        if slide is not None:
            self.slide = slide
        self.group_elements = group_elements
        self.group_element_id = group_element_id
        if group_element is not None:
            self.group_element = group_element
        if hidden is not None:
            self.hidden = hidden
        self.title = title
        if ooxml_id is not None:
            self.ooxml_id = ooxml_id
        self.svg_blob_url = svg_blob_url
        self.base_element_blob_url = base_element_blob_url
        self.changed_base_element_blob_url = changed_base_element_blob_url
        self.package_uri = package_uri
        self.name = name
        if id is not None:
            self.id = id
        if date_created is not None:
            self.date_created = date_created
        if user_created is not None:
            self.user_created = user_created
        if date_modified is not None:
            self.date_modified = date_modified
        if user_modified is not None:
            self.user_modified = user_modified

    @property
    def slide_id(self):
        """Gets the slide_id of this SlideShapeTreesDetails.  # noqa: E501


        :return: The slide_id of this SlideShapeTreesDetails.  # noqa: E501
        :rtype: str
        """
        return self._slide_id

    @slide_id.setter
    def slide_id(self, slide_id):
        """Sets the slide_id of this SlideShapeTreesDetails.


        :param slide_id: The slide_id of this SlideShapeTreesDetails.  # noqa: E501
        :type: str
        """

        self._slide_id = slide_id

    @property
    def slide(self):
        """Gets the slide of this SlideShapeTreesDetails.  # noqa: E501


        :return: The slide of this SlideShapeTreesDetails.  # noqa: E501
        :rtype: SlideSlidesDetails
        """
        return self._slide

    @slide.setter
    def slide(self, slide):
        """Sets the slide of this SlideShapeTreesDetails.


        :param slide: The slide of this SlideShapeTreesDetails.  # noqa: E501
        :type: SlideSlidesDetails
        """

        self._slide = slide

    @property
    def group_elements(self):
        """Gets the group_elements of this SlideShapeTreesDetails.  # noqa: E501


        :return: The group_elements of this SlideShapeTreesDetails.  # noqa: E501
        :rtype: list[SlideGroupElementsDetails]
        """
        return self._group_elements

    @group_elements.setter
    def group_elements(self, group_elements):
        """Sets the group_elements of this SlideShapeTreesDetails.


        :param group_elements: The group_elements of this SlideShapeTreesDetails.  # noqa: E501
        :type: list[SlideGroupElementsDetails]
        """

        self._group_elements = group_elements

    @property
    def group_element_id(self):
        """Gets the group_element_id of this SlideShapeTreesDetails.  # noqa: E501


        :return: The group_element_id of this SlideShapeTreesDetails.  # noqa: E501
        :rtype: str
        """
        return self._group_element_id

    @group_element_id.setter
    def group_element_id(self, group_element_id):
        """Sets the group_element_id of this SlideShapeTreesDetails.


        :param group_element_id: The group_element_id of this SlideShapeTreesDetails.  # noqa: E501
        :type: str
        """

        self._group_element_id = group_element_id

    @property
    def group_element(self):
        """Gets the group_element of this SlideShapeTreesDetails.  # noqa: E501


        :return: The group_element of this SlideShapeTreesDetails.  # noqa: E501
        :rtype: SlideGroupElementsDetails
        """
        return self._group_element

    @group_element.setter
    def group_element(self, group_element):
        """Sets the group_element of this SlideShapeTreesDetails.


        :param group_element: The group_element of this SlideShapeTreesDetails.  # noqa: E501
        :type: SlideGroupElementsDetails
        """

        self._group_element = group_element

    @property
    def hidden(self):
        """Gets the hidden of this SlideShapeTreesDetails.  # noqa: E501


        :return: The hidden of this SlideShapeTreesDetails.  # noqa: E501
        :rtype: bool
        """
        return self._hidden

    @hidden.setter
    def hidden(self, hidden):
        """Sets the hidden of this SlideShapeTreesDetails.


        :param hidden: The hidden of this SlideShapeTreesDetails.  # noqa: E501
        :type: bool
        """

        self._hidden = hidden

    @property
    def title(self):
        """Gets the title of this SlideShapeTreesDetails.  # noqa: E501


        :return: The title of this SlideShapeTreesDetails.  # noqa: E501
        :rtype: str
        """
        return self._title

    @title.setter
    def title(self, title):
        """Sets the title of this SlideShapeTreesDetails.


        :param title: The title of this SlideShapeTreesDetails.  # noqa: E501
        :type: str
        """

        self._title = title

    @property
    def ooxml_id(self):
        """Gets the ooxml_id of this SlideShapeTreesDetails.  # noqa: E501


        :return: The ooxml_id of this SlideShapeTreesDetails.  # noqa: E501
        :rtype: int
        """
        return self._ooxml_id

    @ooxml_id.setter
    def ooxml_id(self, ooxml_id):
        """Sets the ooxml_id of this SlideShapeTreesDetails.


        :param ooxml_id: The ooxml_id of this SlideShapeTreesDetails.  # noqa: E501
        :type: int
        """

        self._ooxml_id = ooxml_id

    @property
    def svg_blob_url(self):
        """Gets the svg_blob_url of this SlideShapeTreesDetails.  # noqa: E501


        :return: The svg_blob_url of this SlideShapeTreesDetails.  # noqa: E501
        :rtype: str
        """
        return self._svg_blob_url

    @svg_blob_url.setter
    def svg_blob_url(self, svg_blob_url):
        """Sets the svg_blob_url of this SlideShapeTreesDetails.


        :param svg_blob_url: The svg_blob_url of this SlideShapeTreesDetails.  # noqa: E501
        :type: str
        """

        self._svg_blob_url = svg_blob_url

    @property
    def base_element_blob_url(self):
        """Gets the base_element_blob_url of this SlideShapeTreesDetails.  # noqa: E501


        :return: The base_element_blob_url of this SlideShapeTreesDetails.  # noqa: E501
        :rtype: str
        """
        return self._base_element_blob_url

    @base_element_blob_url.setter
    def base_element_blob_url(self, base_element_blob_url):
        """Sets the base_element_blob_url of this SlideShapeTreesDetails.


        :param base_element_blob_url: The base_element_blob_url of this SlideShapeTreesDetails.  # noqa: E501
        :type: str
        """

        self._base_element_blob_url = base_element_blob_url

    @property
    def changed_base_element_blob_url(self):
        """Gets the changed_base_element_blob_url of this SlideShapeTreesDetails.  # noqa: E501


        :return: The changed_base_element_blob_url of this SlideShapeTreesDetails.  # noqa: E501
        :rtype: str
        """
        return self._changed_base_element_blob_url

    @changed_base_element_blob_url.setter
    def changed_base_element_blob_url(self, changed_base_element_blob_url):
        """Sets the changed_base_element_blob_url of this SlideShapeTreesDetails.


        :param changed_base_element_blob_url: The changed_base_element_blob_url of this SlideShapeTreesDetails.  # noqa: E501
        :type: str
        """

        self._changed_base_element_blob_url = changed_base_element_blob_url

    @property
    def package_uri(self):
        """Gets the package_uri of this SlideShapeTreesDetails.  # noqa: E501


        :return: The package_uri of this SlideShapeTreesDetails.  # noqa: E501
        :rtype: str
        """
        return self._package_uri

    @package_uri.setter
    def package_uri(self, package_uri):
        """Sets the package_uri of this SlideShapeTreesDetails.


        :param package_uri: The package_uri of this SlideShapeTreesDetails.  # noqa: E501
        :type: str
        """

        self._package_uri = package_uri

    @property
    def name(self):
        """Gets the name of this SlideShapeTreesDetails.  # noqa: E501


        :return: The name of this SlideShapeTreesDetails.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this SlideShapeTreesDetails.


        :param name: The name of this SlideShapeTreesDetails.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def id(self):
        """Gets the id of this SlideShapeTreesDetails.  # noqa: E501


        :return: The id of this SlideShapeTreesDetails.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this SlideShapeTreesDetails.


        :param id: The id of this SlideShapeTreesDetails.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def date_created(self):
        """Gets the date_created of this SlideShapeTreesDetails.  # noqa: E501


        :return: The date_created of this SlideShapeTreesDetails.  # noqa: E501
        :rtype: datetime
        """
        return self._date_created

    @date_created.setter
    def date_created(self, date_created):
        """Sets the date_created of this SlideShapeTreesDetails.


        :param date_created: The date_created of this SlideShapeTreesDetails.  # noqa: E501
        :type: datetime
        """

        self._date_created = date_created

    @property
    def user_created(self):
        """Gets the user_created of this SlideShapeTreesDetails.  # noqa: E501


        :return: The user_created of this SlideShapeTreesDetails.  # noqa: E501
        :rtype: str
        """
        return self._user_created

    @user_created.setter
    def user_created(self, user_created):
        """Sets the user_created of this SlideShapeTreesDetails.


        :param user_created: The user_created of this SlideShapeTreesDetails.  # noqa: E501
        :type: str
        """

        self._user_created = user_created

    @property
    def date_modified(self):
        """Gets the date_modified of this SlideShapeTreesDetails.  # noqa: E501


        :return: The date_modified of this SlideShapeTreesDetails.  # noqa: E501
        :rtype: datetime
        """
        return self._date_modified

    @date_modified.setter
    def date_modified(self, date_modified):
        """Sets the date_modified of this SlideShapeTreesDetails.


        :param date_modified: The date_modified of this SlideShapeTreesDetails.  # noqa: E501
        :type: datetime
        """

        self._date_modified = date_modified

    @property
    def user_modified(self):
        """Gets the user_modified of this SlideShapeTreesDetails.  # noqa: E501


        :return: The user_modified of this SlideShapeTreesDetails.  # noqa: E501
        :rtype: str
        """
        return self._user_modified

    @user_modified.setter
    def user_modified(self, user_modified):
        """Sets the user_modified of this SlideShapeTreesDetails.


        :param user_modified: The user_modified of this SlideShapeTreesDetails.  # noqa: E501
        :type: str
        """

        self._user_modified = user_modified

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, SlideShapeTreesDetails):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other
